
# coding: utf-8

# In[19]:


#########################################################################
# Simple recurrent neural network for classifying words: Chararter Model#
#########################################################################
import numpy as np
from keras.models import Sequential
from keras.layers import Dense, Dropout, Embedding,SimpleRNN,Activation
from keras.layers.embeddings import Embedding
from keras.preprocessing import sequence
from keras.utils import np_utils


# In[20]:

filenames = ['conll2003/en/pos/train.txt','conll2003/en/pos/valid.txt','conll2003/en/pos/test.txt']
char2idx = {}
idx2char = {}


# In[21]:

# read the data
def load_data(path):
    words = []
    labels = []
    for line in open(path, 'r'):
        line = line.strip()
        if len(line) == 0:
            continue
        splits = line.split('\t')
        words.append(splits[0])
        labels.append(splits[1])
    return words,labels

# print the first lines from the traing data
word,label = load_data(filenames[0])
print word[0],label[0]


# In[22]:

# Ugly can be factored 
def getLabels(filepath):
    labels = []
    for line in open(filepath, 'r'):
        line = line.strip()
        if len(line) == 0:
            continue
        splits = line.split('\t')
        labels.append(splits[1].strip())
    return list(set(labels))


# In[23]:

def get_all_chars(fnames):
    words = set()
    for filename in fnames:
        for line in open(filename):
            line = line.strip()
            if len(line) == 0:
                continue
            splits = line.split('\t')
            words.add(splits[0])
    char_list = list(set(''.join(words)))
    char2idx = dict((c, i) for i, c in enumerate(char_list))
    idx2char = dict((i, c) for i, c in enumerate(char_list))
    max_length = 0
    for n in words:
        if len(n) > max_length:
            max_length = len(n)
    if max_length < 50:
        max_length = 50
    return char2idx,idx2char,max_length


# In[24]:

char2idx,idx2char,MAX_LENGTH = get_all_chars(filenames)
labels_list=getLabels(filenames[1])
label2idx = dict((l, i) for i, l in enumerate(labels_list))
idx2Label = dict((i, l) for i, l in enumerate(labels_list))


# In[25]:

def word2chars(name):
     word_chars = list(name)
     word_chars_indices = list(map(lambda char: char2idx.get(char), word_chars))
     return sequence.pad_sequences([word_chars_indices], maxlen=MAX_LENGTH)[0]


# In[26]:

def vectorize_char(cs_w,cs_l):
    X = []
    y = []
    for n, l in zip(cs_w, cs_l):
        X.append(word2chars(n))
        y.append(label2idx.get(l))
    X = np.array(X).astype(np.uint8)
    y = np_utils.to_categorical(np.array(y)).astype(np.bool)
    return X,y


# In[27]:

train_words,train_labels = load_data(filenames[0])
dev_words,dev_labels = load_data(filenames[1])
train_X_char,train_y_char = vectorize_char(train_words,train_labels)
dev_X_char,dev_y_char = vectorize_char(dev_words,dev_labels)

print dev_X_char[0]
print dev_words[0]


# In[28]:

#### Train RNN classifier ####

batch_size = 150
nb_epoch = 1
model = Sequential()
model.add(Embedding(len(char2idx), 50, input_length=MAX_LENGTH, mask_zero=True))
model.add(SimpleRNN(50,return_sequences=False))
model.add(Dense(len(labels_list)))
model.add(Activation('softmax'))
model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=["accuracy"])
model.summary()
model.fit(train_X_char, train_y_char,
                 batch_size=batch_size,
                 epochs=nb_epoch,
                 verbose=1,
                 shuffle=True,
                 validation_data=[dev_X_char, dev_y_char])


# In[29]:

# evaluate the model
score = model.evaluate(dev_X_char, dev_y_char,batch_size=300, verbose=0)
print('Test score:', score[0])
print('Test accuracy:', score[1])

